#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Nielsen"
__doc__ = r"""

           Created on 24-02-2021
           """

import numpy
from matplotlib import pyplot
from matplotlib.pyplot import legend

from draugr.visualisation import (
    MonoChromeStyleSession,
    auto_post_hatch,
    monochrome_hatch_cycler,
    simple_hatch_cycler,
    use_monochrome_style,
)


def line_plot():
    with MonoChromeStyleSession():
        fig, ax = pyplot.subplots(1, 1)
        for x in range(3):
            import numpy

            ax.plot(numpy.random.rand(10), label=f"{x}")

    from matplotlib.pyplot import legend

    legend()
    pyplot.show()


def bar_plot():
    use_monochrome_style(prop_cycler=monochrome_hatch_cycler)

    fig, ax = pyplot.subplots(1, 1)

    for x in range(3):
        ax.bar(x, numpy.random.randint(2, 10), label=f"{x}")

    legend()
    from draugr.visualisation.matplotlib_utilities.quirks import fix_edge_gridlines

    fix_edge_gridlines(ax)
    auto_post_hatch(ax, simple_hatch_cycler)
    pyplot.show()


if __name__ == "__main__":
    line_plot()
    bar_plot()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Nielsen"
__doc__ = r"""

           Created on 25-05-2021
           """

from .formatting import *
from .from_dict import *
from .latex_mean_std import *
from .misc_utilities import *
from .multi_index_utilities import *
from .styling import *



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Nielsen"
__doc__ = r"""

           Created on 22/06/2020
           """

import os
from pathlib import Path

from apppath import AppPath, ensure_existence
from draugr.tensorboard_utilities import TensorboardEventExporter
from draugr.tqdm_utilities import progress_bar
from draugr.writers import (
    TestingCurves,
    TestingScalars,
    TrainingCurves,
    TrainingScalars,
)

__all__ = ["extract_scalars_as_csv", "extract_tensors_as_csv", "extract_metrics"]

EXPORT_RESULTS_PATH = Path.cwd()


def extract_scalars_as_csv(
    train_path: Path = EXPORT_RESULTS_PATH / "csv" / "training",
    test_path: Path = EXPORT_RESULTS_PATH / "csv" / "testing",
    export_train: bool = True,
    export_test: bool = True,
    verbose: bool = False,
    only_extract_from_latest_event_file: bool = False,
) -> None:
    """
    :param train_path:
    :param test_path:
    :param export_train:
    :param export_test:
    :param verbose:
    :param only_extract_from_latest_event_file:
    """
    if only_extract_from_latest_event_file:
        max_load_time = max(
            list(
                AppPath(
                    "Adversarial Speech", "Christian Heider Nielsen"
                ).user_log.iterdir()
            ),
            key=os.path.getctime,
        )
        unique_event_files_parents = set(
            [ef.parent for ef in max_load_time.rglob("events.out.tfevents.*")]
        )
        event_files = {max_load_time: unique_event_files_parents}
    else:
        event_files = {
            a: set([ef.parent for ef in a.rglob("events.out.tfevents.*")])
            for a in list(
                AppPath(
                    "Adversarial Speech", "Christian Heider Nielsen"
                ).user_log.iterdir()
            )
        }

    for k, v in progress_bar(event_files.items()):
        for e in progress_bar(v):
            relative_path = e.relative_to(k)
            mapping_id, *rest = relative_path.parts
            mappind_id_test = f"{mapping_id}_Test_{relative_path.name}"
            # model_id = relative_path.parent.name can be include but is always the same
            relative_path = Path(*(mappind_id_test, *rest))
            with TensorboardEventExporter(e, save_to_disk=True) as tee:
                if export_test:
                    out_tags = []
                    for tag in progress_bar(TestingScalars):
                        if tag.value in tee.available_scalars:
                            out_tags.append(tag.value)

                    if len(out_tags):
                        tee.scalar_export_csv(
                            *out_tags,
                            out_dir=ensure_existence(
                                test_path / k.name / relative_path,
                                force_overwrite=True,
                                verbose=verbose,
                            ),
                        )
                        print(e)
                    else:
                        if verbose:
                            print(
                                f"{e}, no requested tags found {TestingScalars.__members__.values()}, {tee.available_scalars}"
                            )

                if export_train:
                    out_tags = []
                    for tag in progress_bar(TrainingScalars):
                        if tag.value in tee.available_scalars:
                            out_tags.append(tag.value)

                    if len(out_tags):
                        tee.scalar_export_csv(
                            *out_tags,
                            out_dir=ensure_existence(
                                train_path / k.name / relative_path,
                                force_overwrite=True,
                                verbose=verbose,
                            ),
                        )
                    else:
                        if verbose:
                            print(
                                f"{e}, no requested tags found {TrainingScalars.__members__.values()}, {tee.available_scalars}"
                            )


def extract_tensors_as_csv(
    train_path: Path = EXPORT_RESULTS_PATH / "csv" / "training",
    test_path: Path = EXPORT_RESULTS_PATH / "csv" / "testing",
    export_train: bool = False,
    export_test: bool = True,
    verbose: bool = False,
    only_extract_from_latest_event_file: bool = False,
) -> None:
    """

    :param train_path:
    :param test_path:
    :param export_train:
    :param export_test:
    :param verbose:
    :param only_extract_from_latest_event_file:
    :return:"""
    if only_extract_from_latest_event_file:
        max_load_time = max(
            list(
                AppPath(
                    "Adversarial Speech", "Christian Heider Nielsen"
                ).user_log.iterdir()
            ),
            key=os.path.getctime,
        )
        unique_event_files_parents = set(
            [ef.parent for ef in max_load_time.rglob("events.out.tfevents.*")]
        )
        event_files = {max_load_time: unique_event_files_parents}
    else:
        event_files = {
            a: set([ef.parent for ef in a.rglob("events.out.tfevents.*")])
            for a in list(
                AppPath(
                    "Adversarial Speech", "Christian Heider Nielsen"
                ).user_log.iterdir()
            )
        }

    for k, v in progress_bar(event_files.items()):
        for e in progress_bar(v):
            relative_path = e.relative_to(k)
            mapping_id, *rest = relative_path.parts
            mapping_id_test = f"{mapping_id}_Test_{relative_path.name}"
            # model_id = relative_path.parent.name can be include but is always the same
            relative_path = Path(*(mapping_id_test, *rest))
            with TensorboardEventExporter(e, save_to_disk=True) as tee:
                if export_test:
                    out_tags = []
                    for tag in progress_bar(TestingCurves):
                        if tag.value in tee.available_tensors:
                            out_tags.append(tag.value)

                    if len(out_tags):
                        tee.pr_curve_export_csv(
                            *out_tags,
                            out_dir=ensure_existence(
                                test_path / k.name / relative_path,
                                force_overwrite=True,
                                verbose=verbose,
                            ),
                        )
                    else:
                        if verbose:
                            print(
                                f"{e}, no requested tags found {TestingCurves.__members__.values()}, {tee.available_tensors}"
                            )

                if export_train:  # TODO: OUTPUT for all epoch steps, no support yet
                    out_tags = []
                    for tag in progress_bar(TrainingCurves):
                        if tag.value in tee.available_tensors:
                            out_tags.append(tag.value)

                    if len(out_tags):
                        tee.pr_curve_export_csv(
                            *out_tags,
                            out_dir=ensure_existence(
                                # train_path / max_load_time.name / relative_path, # MAX LOAD TIME HERE?
                                train_path
                                / k.name
                                / relative_path,  # MAX LOAD TIME HERE?
                                force_overwrite=True,
                                verbose=verbose,
                            ),
                        )
                    else:
                        if verbose:
                            print(
                                f"{e}, no requested tags found {TrainingCurves.__members__.values()}, {tee.available_tensors}"
                            )


def extract_metrics(only_extract_latest=False):
    extract_scalars_as_csv(only_extract_from_latest_event_file=only_extract_latest)
    extract_tensors_as_csv(only_extract_from_latest_event_file=only_extract_latest)


if __name__ == "__main__":
    extract_metrics(only_extract_latest=True)
    # extract_scalars_as_csv(verbose=False,export_train=False)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Nielsen"
__doc__ = r"""

           Created on 28-03-2021
           """

import time

import numpy

from draugr import PROJECT_APP_PATH
from draugr.torch_utilities import (
    TensorBoardPytorchWriter,
    to_tensor,
)
from draugr.torch_utilities.tensors.dimension_order import (
    nhwc_to_nchw_tensor,
)
from draugr.torch_utilities.writers.tensorboard.tensorboard_pytorch_writer import (
    VideoInputDimsEnum,
)

if __name__ == "__main__":

    def main():
        """ """
        import gym

        env = gym.make("Pendulum-v0")
        state = env.reset()

        with TensorBoardPytorchWriter(
            PROJECT_APP_PATH.user_log / "Tests" / "Writers"
        ) as writer:
            frames = []
            done = False

            start = time.time()
            while not done:
                frames.append(env.render("rgb_array"))
                state, reward, done, info = env.step(env.action_space.sample())
            fps = len(frames) / (time.time() - start)

            env.close()
            video_array = numpy.array(frames)
            writer.video(
                "replay05",
                nhwc_to_nchw_tensor(to_tensor(video_array)).unsqueeze(0),
                frame_rate=fps,
            )
            writer.video(
                "replay06",
                video_array,
                0,
                input_dims=VideoInputDimsEnum.thwc,
                frame_rate=fps,
            )
            writer.video(
                "replay08",
                numpy.stack([video_array, video_array]),
                0,
                input_dims=VideoInputDimsEnum.nthwc,
                frame_rate=fps,
            )

    main()



#  Copyright (c) 2021. Lorem ipsum dolor sit amet, consectetur adipiscing elit.
#  Morbi non lorem porttitor neque feugiat blandit. Ut vitae ipsum eget quam lacinia accumsan.
#  Etiam sed turpis ac ipsum condimentum fringilla. Maecenas magna.
#  Proin dapibus sapien vel ante. Aliquam erat volutpat. Pellentesque sagittis ligula eget metus.
#  Vestibulum commodo. Ut rhoncus gravida arcu.


def asd():
    """ """
    import time

    from draugr import IgnoreInterruptSignal, busy_indicator

    next_reading = time.time()
    with IgnoreInterruptSignal():
        print("Publisher started")

        for _ in busy_indicator():
            next_reading += 3
            sleep_time = next_reading - time.time()

            if sleep_time > 0:
                time.sleep(sleep_time)


if __name__ == "__main__":

    if False:
        from pathlib import Path

        from draugr.os_utilities.linux_utilities.systemd_utilities.service_management import (
            install_service,
        )

        install_service(Path(__file__), "busy_script")
    else:
        asd()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Nielsen"
__doc__ = r"""

           Created on 14-02-2021
           """

import matlab
import numpy

from draugr.matlab_utilities import dict_to_sparse, matlab_to_ndarray, ndarray_to_matlab


def asiduj_test():
    """
    Test of the module

    It runs the doctest and create other tests with matlab engine calls."""
    import scipy.linalg as spl

    print("Run matlab engine...")
    if len(matlab.engine.find_matlab()) == 0:
        # si aucune session share, run
        eng = matlab.engine.start_matlab()
    else:
        # connect to a session
        eng = matlab.engine.connect_matlab(matlab.engine.find_matlab()[0])
        print("connected...")

    print("Further tests....\n")

    # create matlab data
    # ------------------------------------------------------------------------
    mf = eng.rand(3)
    mc = matlab.double([[1 + 1j, 0.3, 1j], [1.2j - 1, 0, 1 + 1j]], is_complex=True)
    mi64 = matlab.int64([1, 2, 3])
    mi8 = matlab.int8([1, 2, 3])
    mb = matlab.logical([True, True, False])

    # Test conversion from matlab to numpy
    # ------------------------------------------------------------------------
    npf = matlab_to_ndarray(mf)  # no copy, if mf is changed, npf change!
    npc = matlab_to_ndarray(mc)  # still copy for complex (only)
    npi64 = matlab_to_ndarray(mi64)
    npi8 = matlab_to_ndarray(mi8)
    npb = matlab_to_ndarray(mb)

    # Test conversion from numpy to matlab
    # ------------------------------------------------------------------------
    npi = numpy.array(
        [[1, 2, 3], [4, 5, 6], [7, 8, 9], [10, 11, 12]], dtype=numpy.int64, order="F"
    )
    mi = ndarray_to_matlab(npi)
    mc2 = ndarray_to_matlab(npc)
    mf2 = ndarray_to_matlab(npf)  # copy, because npf has 'F' order (comes from mlarray)
    mi64_2 = ndarray_to_matlab(npi)
    mb2 = ndarray_to_matlab(npb)

    # test orientation in the matlab workspace
    # ------------------------------------------------------------------------
    eng.workspace["mi"] = mi64_2
    eng.workspace["mc2"] = mc2

    # check results
    # ------------------------------------------------------------------------
    npcc = numpy.array(
        [
            [1.0, 1.1 + 1j],
            [
                1.12 + 0.13j,
                22.1,
            ],
        ],
        dtype=numpy.complex,
    )  # assume C
    mcc = ndarray_to_matlab(npcc)
    npcc_inv = spl.inv(npcc)
    mcc_inv = eng.inv(mcc)
    print("Are the inverse of matrix equal ?")
    print(mcc_inv)
    print(npcc_inv)

    #    # no copy check
    #    # ------------------------------------------------------------------------
    #    # complex
    #
    #    npcc[0,0]=0.25
    #    print("Are the data reuse ?", ", OWNDATA =", mcc._real.flags.owndata,
    #          "same base =", mcc._real.base is npcc,
    #          ', If one is modified, the other is modified =', mcc._real[0]==npcc[0,0])
    #

    # test sparse matrix requiert Recast4py.m
    K1, K2 = eng.sptest(3.0, nargout=2)
    Ksp1 = dict_to_sparse(K1)
    Ksp2 = dict_to_sparse(K2)


def doctest_test():
    """run test procedure with doctest"""
    import doctest

    # invoke the testmod function to run tests contained in docstring
    stats = doctest.testmod()
    print(stats)
    return stats



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Nielsen"
__doc__ = r"""

           Created on 04-11-2020
           """

import csv
import os
import re
from pathlib import Path

import numpy
import pandas as pd
import torch
import torchvision.models as models
import torchvision.transforms as transforms
from PIL import Image

if __name__ == "__main__":

    def main(im_path="images"):
        """ """

        def get_vector(input_image):
            """ """
            image = input_image.convert(
                "RGB"
            )  # in case input image is not in RGB format
            img_t = transform(image)
            batch_t = torch.unsqueeze(img_t, 0)
            my_embedding = torch.zeros([1, 512, 1, 1])

            def copy_data(m, i, o):
                """ """
                my_embedding.copy_(o.data)

            h = layer.register_forward_hook(copy_data)
            model(batch_t)
            h.remove()
            return my_embedding.squeeze().cpu().numpy()

        model = models.resnet18(pretrained=True)
        layer = model._modules.get("avgpool")
        model.eval()
        transform = transforms.Compose(
            [
                transforms.Resize(256),
                transforms.CenterCrop(224),
                transforms.ToTensor(),
                transforms.Normalize(
                    mean=[0.485, 0.456, 0.406], std=[0.229, 0.224, 0.225]
                ),
            ]
        )

        im_names = [
            str(Path(root) / name)
            for root, dirs, files in os.walk(im_path)
            for name in files
            if name.endswith(".jpg")
        ]
        existing_images_df = pd.DataFrame(
            [re.findall(r"[\w']+", im_name)[1:3] for im_name in im_names],
            columns=["cat_id", "pid"],
        )
        existing_images_df["impath"] = im_names
        vecs = [
            list(get_vector(Image.open(impath)))
            for _, pid, impath in existing_images_df.values
        ]

        with open("vis/feature_vecs.tsv", "w") as fw:
            csv_writer = csv.writer(fw, delimiter="\t")
            csv_writer.writerows(vecs)

        images = [
            Image.open(filename).resize((300, 300))
            for filename in existing_images_df["impath"]
        ]
        image_width, image_height = images[0].size
        one_square_size = int(numpy.ceil(numpy.sqrt(len(images))))
        master_width = image_width * one_square_size
        master_height = image_height * one_square_size

        spriteimage = Image.new(
            mode="RGBA", size=(master_width, master_height), color=(0, 0, 0, 0)
        )  # fully transparent

        for count, image in enumerate(images):
            div, mod = divmod(count, one_square_size)
            h_loc = image_width * div
            w_loc = image_width * mod
            spriteimage.paste(image, (w_loc, h_loc))

        spriteimage.convert("RGB").save("sprite.jpg", transparency=0)

        metadata = existing_images_df[["cat_id", "pid"]]
        metadata.to_csv("vis/metadata.tsv", sep="\t", index=False)

    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Nielsen"
__doc__ = r"""

           Created on 17-02-2021
           """

from matplotlib import cycler, pyplot, rcParams
from matplotlib.axes import Axes

from .styles import simple_hatch_cycler

__all__ = ["fix_edge_gridlines", "auto_post_print_dpi", "auto_post_hatch"]

from warg import Number


def fix_edge_gridlines(
    ax: Axes = None,
) -> None:  # TODO: make a wrapper version of the function
    """
    Fixes gridlines when using zero margins round_number number ticks

    :param ax:
    :return:"""
    if ax is None:
        ax = pyplot.gca()
    ax.xaxis.get_gridlines()[-1].set_clip_on(False)  # last gridline
    ax.yaxis.get_gridlines()[0].set_clip_on(False)  # first gridline


def auto_post_print_dpi(scalar: Number = 5) -> None:
    """
    auto scale dpi of lines for print

    :param scalar:
    :return:"""
    dpi = rcParams["figure.dpi"]
    line_width = 1.0 / dpi * scalar
    rcParams["hatch.linewidth"] = line_width * 2
    rcParams["grid.linewidth"] = line_width


def auto_post_hatch(
    ax: Axes = None, hatch_cycler: cycler = simple_hatch_cycler
) -> None:
    """
    Auto hatches patch-types because matplotlib's prop-cycler does not iterate hatch_props

    :param ax:
    :param hatch_cycler:
    :return:"""
    if ax is None:
        ax = pyplot.gca()
    for p, d in zip(ax.patches, hatch_cycler):
        p.set_hatch(next(iter(d.values())))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Nielsen"
__doc__ = r"""

           Created on 08-12-2020
           """

from itertools import tee
from typing import Any, Generator, Iterable

import tqdm
from notus.notification import JobNotificationSession

from warg import drop_unused_kws, passes_kws_to

__all__ = ["progress_bar"]


@drop_unused_kws
@passes_kws_to(tqdm.tqdm)
def progress_bar(
    iterable: Iterable,
    description: str = None,
    *,
    leave: bool = False,
    notifications: bool = False,
    total: int = None,
    auto_total_generator: bool = False,
    auto_describe_iterator: bool = True,  # DOES NOT WORK IS THIS FUNCTION IS ALIAS does not match!
    alias="progress_bar",
    disable: bool = False,
    **kwargs,
) -> Any:
    """
    hint:
    use next first and then later use send instead of next to set a new tqdm description if desired.
    """
    if not disable:
        if description is None and auto_describe_iterator:
            from warg import get_first_arg_name

            description = get_first_arg_name(alias)

        if total is None and isinstance(iterable, Generator) and auto_total_generator:
            iterable, ic = tee(iterable, 2)
            total = len(list(ic))
            if total == 0:
                print(f"WARNING zero length iterable - {description}:{iterable}")

        generator = tqdm.tqdm(
            iterable,
            description,
            leave=leave,
            total=total,
            disable=disable,  # redundant
            **kwargs,
        )
        if notifications:
            with JobNotificationSession(description):
                for val in generator:
                    a = yield val
                    if a:
                        generator.set_description(a)
            return
        for val in generator:
            a = yield val
            if a:
                generator.set_description(a)
    else:
        yield from iterable


if __name__ == "__main__":

    def dsad3123():
        """ """
        from time import sleep

        for a in progress_bar([2.13, 8921.9123, 923], notifications=False):
            sleep(1)

    def asd21sa():
        """ """
        from time import sleep

        pb = progress_bar  # Aliased!

        for a in pb([2.13, 8921.9123, 923], notifications=False):
            sleep(1)

    def dict_items():
        """ """
        from time import sleep

        class exp_v:
            Test_Sets = {v: v for v in range(9)}

        for a in progress_bar(exp_v.Test_Sets.items()):
            sleep(1)

    def send_example():
        from itertools import count

        pb = progress_bar(count())
        next(pb)
        for a in range(100000):
            pb.send(f"step_{a}")

    # dsad3123()
    # asd21sa()
    # dict_items()
    send_example()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Nielsen"
__doc__ = r"""

           Created on 28-03-2021
           """

import torch

from draugr import PROJECT_APP_PATH
from draugr.torch_utilities import (
    TensorBoardPytorchWriter,
)

if __name__ == "__main__":

    def main():
        """
        TODO:!!!!"""

        with TensorBoardPytorchWriter(
            PROJECT_APP_PATH.user_log / "Tests" / "Writers"
        ) as writer:
            vertices_tensor = torch.as_tensor(
                [
                    [1, 1, 1],
                    [-1, -1, 1],
                    [1, -1, -1],
                    [-1, 1, -1],
                ],
                dtype=torch.float,
            ).unsqueeze(0)
            colors_tensor = torch.as_tensor(
                [
                    [255, 0, 0],
                    [0, 255, 0],
                    [0, 0, 255],
                    [255, 0, 255],
                ],
                dtype=torch.int,
            ).unsqueeze(0)
            faces_tensor = torch.as_tensor(
                [
                    [0, 2, 3],
                    [0, 3, 1],
                    [0, 1, 2],
                    [1, 3, 2],
                ],
                dtype=torch.int,
            ).unsqueeze(0)

            writer.mesh(
                "my_mesh", vertices_tensor, colors=colors_tensor, faces=faces_tensor
            )

    main()



import cmd
from turtle import *


class TurtleShell(cmd.Cmd):
    intro = "Welcome to the turtle shell.   Type help or ? to list commands.\n"
    prompt = "(turtle) "
    file = None

    # ----- basic turtle commands -----
    def do_forward(self, arg):
        """Move the turtle forward by the specified distance:  FORWARD 10"""
        forward(*parse(arg))

    def do_right(self, arg):
        """Turn turtle right by given number of degrees:  RIGHT 20"""
        right(*parse(arg))

    def do_left(self, arg):
        """Turn turtle left by given number of degrees:  LEFT 90"""
        left(*parse(arg))

    def do_goto(self, arg):
        """Move turtle to an absolute position with changing orientation.  GOTO 100 200"""
        goto(*parse(arg))

    def do_home(self, arg):
        """Return turtle to the home position:  HOME"""
        home()

    def do_circle(self, arg):
        """Draw circle with given radius an options extent and steps:  CIRCLE 50"""
        circle(*parse(arg))

    def do_position(self, arg):
        """Print the current turtle position:  POSITION"""
        print(f"Current position is {position()}\n")

    def do_heading(self, arg):
        """Print the current turtle heading in degrees:  HEADING"""
        print(f"Current heading is {heading()}\n")

    def do_color(self, arg):
        """Set the color:  COLOR BLUE"""
        color(arg.lower())

    def do_undo(self, arg):
        """Undo (repeatedly) the last turtle action(s):  UNDO"""

    def do_reset(self, arg):
        """Clear the screen and return turtle to center:  RESET"""
        reset()

    def do_bye(self, arg):
        """Stop recording, close the turtle window, and exit:  BYE"""
        print("Thank you for using Turtle")
        self.close()
        bye()
        return True

    # ----- record and playback -----
    def do_record(self, arg):
        """Save future commands to filename:  RECORD rose.cmd"""
        self.file = open(arg, "w")

    def do_playback(self, arg):
        """Playback commands from a file:  PLAYBACK rose.cmd"""
        self.close()
        with open(arg) as f:
            self.cmdqueue.extend(f.read().splitlines())

    def precmd(self, line):
        line = line.lower()
        if self.file and "playback" not in line:
            print(line, file=self.file)
        return line

    def close(self):
        if self.file:
            self.file.close()
            self.file = None


def parse(arg):
    """Convert a series of zero or more numbers to an argument tuple"""
    return tuple(map(int, arg.split()))


if __name__ == "__main__":
    TurtleShell().cmdloop()



if __name__ == "__main__":
    from scripts import busy_script

    # print()

    # print(sh.ls("/home/heider"))
    # print(sys.executable)

    # print(sh.systemctl('status', 'lightdm.service'))

    from pathlib import Path

    from draugr.os_utilities.linux_utilities.systemd_utilities.service_management import (
        install_service,
        RunAsEnum,
        remove_service,
    )

    remove_service("busy_script", run_as=RunAsEnum.root)
    # install_service(Path(busy_script.__file__), "busy_script",run_as=RunAsEnum.root)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = "Christian Heider Nielsen"
__doc__ = r"""

           Created on 12-05-2021
           """

__all__ = ["min_interval_wrapper", "min_interval_wrapper_global"]


def min_interval_wrapper(f: callable, min_interval: int = 100) -> callable:
    """
    to ensure that a function is now being called more often than max_freq, TODO: use proper naming for the interval
    :param f:
    :param min_interval:
    :return:
    """

    def s(last_exec, *, step_i, verbose: bool = False, **kwargs) -> int:
        if verbose:
            print(f"{f, last_exec, step_i, min_interval}")
        if step_i - last_exec >= min_interval:
            f(step_i=step_i, verbose=verbose, **kwargs)
            return step_i
        return last_exec

    return s


_GLOBAL_COUNTERS = {}


def min_interval_wrapper_global(f: callable, min_interval: int = 100) -> callable:
    """
    to ensure that a function is now being called more often than max_freq, TODO: use proper naming for the interval
    :param f:
    :param min_interval:
    :return:
    """

    _GLOBAL_COUNTERS[f] = 0

    def s(*, step_i, verbose: bool = False, **kwargs) -> None:
        if verbose:
            print(f"{f, _GLOBAL_COUNTERS[f], step_i, min_interval}")
        if step_i - _GLOBAL_COUNTERS[f] >= min_interval:
            _GLOBAL_COUNTERS[f] = step_i
            f(step_i=step_i, verbose=verbose, **kwargs)

    return s


if __name__ == "__main__":

    def a(step_i, **kwargs):
        print(step_i)

    def uhsud():
        b = min_interval_wrapper(a)
        c = 0
        for i in range(1000 + 1):
            c = b(c, step_i=i)

    def uhsud23():
        from random import random

        b = min_interval_wrapper_global(a)

        for i in range(1000 + 1):
            if random() > 0.8:
                b(step_i=i)

    def uhsud123():
        from random import random

        b = min_interval_wrapper_global(a, 0)

        for i in range(1000 + 1):
            if random() > 0.8 or True:
                b(step_i=i)

    uhsud123()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import numpy
import pytest

from draugr.drawers.mpl_drawers.distribution_plot import DistributionPlot

__author__ = "Christian Heider Nielsen"
__doc__ = r"""

           Created on 9/2/19
           """


@pytest.mark.skip
def test_moving_distribution_plot():
    data_generator = iter(
        [
            [1, 0, 0],
            [1, 0, 0],
            [1, 0, 0],
            [1, 0, 0],
            [1, 0, 0],
            [0, 0, 1],
            [0, 0, 1],
            [0, 0, 1],
            [0, 0, 1],
            [0, 0, 1],
            [0, 0, 1],
            [0, 1, 0],
            [0, 1, 0],
            [0, 1, 0],
            [0, 1, 0],
            [0, 1, 0],
            [0, 1, 0],
            [0, 1, 0],
            [0, 1, 0],
        ]
    )
    # moving_distribution_plot(data_generator, labels=("a", "b", "c"))
    delta = 1.0 / 60.0

    s = DistributionPlot()
    for LATEST_GPU_STATS in range(100):
        s.draw(numpy.random.sample(), delta)
    assert True


if __name__ == "__main__":
    test_moving_distribution_plot()
